/************************************************************************
 *   MODULE  : INIT_UI.C
 *
 *   PURPOSE : initializes the user interface parameters
 *
 *   AUTHOR  : Ron Hawkins & Craig Woodfolk, SAIC (original)
 *             Kevin Russo, SAIC (1, 3)
 *             Melissa Mathews, SAIC (2)
 *
 *   Change     Date          Reason
 *   ------     ----          ------
 *   Original   25 Sept 92    n/a
 *   1          20 Oct 92     overall program update
 *   2          17 Jun 93     converted xvt 2.0 calls to xvt 3.1
 *   3          22 Jul 93     implemented new TEC LOS algorithm
 *
 ************************************************************************/

/****************
 * header files *
 ****************/

#include "los.h"

/************************************************************************
 *   FUNCTION     : init_ui
 *
 *   PURPOSE      : initialize the user interface parameters
 *
 *   INPUTS       : prod_raster - pointer to the los/profile product
 *
 *   RETURN VALUE : none
 *
 *   CALLED FROM  : do_prod in module DO_PROD.C
 *
 ************************************************************************/

#ifdef PROTO
void init_ui_params( RASTER **tprod_raster )
#else
void init_ui_params( tprod_raster )
RASTER **tprod_raster;
#endif

{
   extern PARAM_CALC     param;               /* calculations location  */
   extern SETTINGS_TYPE  settings;            /* user input parameters  */
   extern RASTER         elev_raster;         /* DTED raster            */

   /* set the radial length */
   param.radial_length = settings.range;
   param.orig_radial_length  = settings.range;

   /* get the product resolution */
   (*tprod_raster)->data_hdr.y_res = elev_raster.data_hdr.y_res;
   (*tprod_raster)->data_hdr.x_res = elev_raster.data_hdr.x_res;

   /* calculate resolution in meters */
   param.y_res_m = (*tprod_raster)->data_hdr.y_res * METERS_PER_DEGREE;
   param.x_res_m = ((*tprod_raster)->data_hdr.x_res * METERS_PER_DEGREE) *
                   cos(settings.observer.loc.y / DEG_PER_RAD);

   /* convert to # of meters per degree */
   param.grid_size_m  = (GFLOAT)minof(param.y_res_m, param.x_res_m);
   param.grid_size_dd = (GFLOAT)minof((*tprod_raster)->data_hdr.y_res,
                                     (*tprod_raster)->data_hdr.x_res);

   if (settings.profile)
   {
      /* performing a profile - only one radial needed */
      param.num_radials = 1;
      param.radial_spacing = 1;
   }
   else
   {
      param.radial_spacing = (GFLOAT) ((atan(param.grid_size_m /
                             (GFLOAT) param.radial_length) * ARCTAN_DEGREES));
      param.num_radials    = (long) ((settings.end - settings.start) /
                             param.radial_spacing + 2);

      /* ensure appropriate radial spacing */
      if (param.num_radials > (long)MAX_RADIALS)
      {
         param.num_radials    = (long)MAX_RADIALS;
         param.radial_spacing = (GFLOAT) ((settings.end - settings.start) /
                                (param.num_radials - 2));
      }
   }

   /* calculate number of heights along the radial */
   param.max_nhts_in_radial = (long)(param.radial_length /
                              param.grid_size_m + ONE);

   /* recalculate radial length */
   param.radial_length = (GFLOAT)(param.grid_size_m * param.max_nhts_in_radial);

   /* calculate product extents (right, left, top, bottom, and center) */
   (*tprod_raster)->data_hdr.extent.right     = settings.observer.loc.x +
                                               (param.radial_length *
                                               (EXTENTS / (METERS_PER_DEGREE *
                                               cos(settings.observer.loc.y /
                                               DEG_PER_RAD))));
   (*tprod_raster)->data_hdr.extent.left      = settings.observer.loc.x -
                                               (param.radial_length *
                                               (EXTENTS / (METERS_PER_DEGREE *
                                               cos(settings.observer.loc.y /
                                               DEG_PER_RAD))));
   (*tprod_raster)->data_hdr.extent.top       = settings.observer.loc.y +
                                               (param.radial_length *
                                               (EXTENTS / METERS_PER_DEGREE));
   (*tprod_raster)->data_hdr.extent.bottom    = settings.observer.loc.y -
                                               (param.radial_length *
                                               (EXTENTS / METERS_PER_DEGREE));

   (*tprod_raster)->data_hdr.center.lon = ((*tprod_raster)->data_hdr.extent.left +
                                         (*tprod_raster)->data_hdr.extent.right)/
                                         (GFLOAT)2;
   (*tprod_raster)->data_hdr.center.lat = ((*tprod_raster)->data_hdr.extent.top +
                                         (*tprod_raster)->data_hdr.extent.bottom)
                                         / (GFLOAT)2;

   /* calculate the number of product elements */
   (*tprod_raster)->data_matrix.width = ((*tprod_raster)->data_hdr.extent.right -
                                       (*tprod_raster)->data_hdr.extent.left) /
                                       (*tprod_raster)->data_hdr.x_res +
                                       (GFLOAT)GRID_VALUE;

   (*tprod_raster)->data_matrix.height = ((*tprod_raster)->data_hdr.extent.top -
                                        (*tprod_raster)->data_hdr.extent.bottom)/
                                        (*tprod_raster)->data_hdr.y_res +
                                        (GFLOAT)GRID_VALUE;

   /* make sure we get an odd number of y matrix values */
   if (fmod((*tprod_raster)->data_matrix.height, (GFLOAT)2) == 0)
   {
      (*tprod_raster)->data_hdr.extent.top +=
                          (*tprod_raster)->data_hdr.y_res / (GFLOAT)2;
      (*tprod_raster)->data_hdr.extent.bottom -=
                          (*tprod_raster)->data_hdr.y_res / (GFLOAT)2;
      (*tprod_raster)->data_matrix.height = 
                          ((*tprod_raster)->data_hdr.extent.top -
                          (*tprod_raster)->data_hdr.extent.bottom) /
                          (*tprod_raster)->data_hdr.y_res + (GFLOAT)GRID_VALUE;
   }

   /* make sure we get an odd number of x matrix values */
   if (fmod((*tprod_raster)->data_matrix.width, (GFLOAT)2) == 0)
   {
      (*tprod_raster)->data_hdr.extent.right +=
                          (*tprod_raster)->data_hdr.x_res / (GFLOAT)2;
      (*tprod_raster)->data_hdr.extent.left -=
                          (*tprod_raster)->data_hdr.x_res / (GFLOAT)2;
      (*tprod_raster)->data_matrix.width =
                          ((*tprod_raster)->data_hdr.extent.right -
                          (*tprod_raster)->data_hdr.extent.left) /
                          (*tprod_raster)->data_hdr.x_res + (GFLOAT)GRID_VALUE;
   }

   /* calculate the number of bytes per row */
   (*tprod_raster)->data_matrix.bytes_per_row = (*tprod_raster)->data_matrix.width
                                               * sizeof(short); 
}
